"""
SpotRM.py - Script for searching for Reactive Metabolite liabilities

Copyright (C) Awametox AB 2021
support@spotrm.com
Version: 0.4.0
Date: 2021-05-17
"""

import app
import PythonQt
import requests
from requests.auth import HTTPBasicAuth
import json
import time
from DataSetUtils import *


class SpotRM:
    def __init__(self, appui):
        self.appui = appui
        self.base_url = "https://www.spotrm.com/api/v1"
        self.header = {"content-type": "application/json"}
        self.time = 0
        try:
            self.token = self._getToken()
        except Exception as e:
            raise e


    def _fetchToken(self):
        login = PythonQt.LogInDialog()
        login.setWindowTitle("SpotRM Login")
        if login.run():
            authData = HTTPBasicAuth(login.getUser(), login.getPassword())
        else:
            raise Exception("Login canceled")
        url = self.base_url + "/tokens/"
        response = requests.post(url, auth=authData)
        if response.status_code == 200:
            self.time = time.time()
            return (json.loads(response.text)["token"], "")
        elif json.loads(response.text)["message"] == "Invalid credentials":
            self.appui.showError(json.loads(response.text)["message"])
            self._fetchToken()
        else:
            print("There was an error: " + json.loads(response.text)["message"])
            self.appui.showError(json.loads(response.text)["message"])
            raise Exception(json.loads(response.text)["message"])


    def _getToken(self):
        now = time.time()
        if now - self.time > 3300:      # Token older than 55m
            self.token = self._fetchToken()
            return self.token
        else:
            return self.token


    def getAlerts(self, smiles):
        url = self.base_url + "/search/smarts/smiles"
        response = requests.post(
            url, data=json.dumps(smiles), headers=self.header, auth=self._getToken()
        )
        try:
            rtn = json.loads(response.text)
        except Exception as e:
            print("There was an error: " + repr(e))
        # Return is a list of lists of alert ID, alert name, and severity
        if isinstance(rtn, dict):  # Errors are dictionaries
            return rtn
        else:
            rtn.sort(key = lambda x: int(x[2]), reverse = True)
            return rtn

    def getDrug(self, alertID):
        url = self.base_url + "/get/drug/smarts_id/" + alertID
        response = requests.get(url, auth=self._getToken())
        try:
            resp_dict = json.loads(response.text)
        except Exception as e:
            print("There was an error: " + repr(e))
        return resp_dict


def get_drug_links(drugs):
    count = 0
    result_list = []
    for x in list(drugs.values()):
        if x["ColourCode"] == "Red" and count < 3:
            if not x["References"]:
                result_list.append("")
                continue
            # Only want one reference from each drug
            rank = ["Mgph", "MiC", "Link"]
            x["References"].sort(key=lambda item: rank.index(item.get("RefType")))
            r = x["References"][0]
            # If reference is MiC or Mgph need to convert to full path.
            if r["RefType"] == "Mgph" or r["RefType"] == "MiC":
                result_list.append(
                    '<a href="http://static.spotrm.com/pdf/'
                    + r["Ref"]
                    + '">'
                    + x["DrugName"]
                    + "</a>"
                )
            else:
                result_list.append(
                    '<a href="'
                    + r["Ref"]
                    + '">'
                    + x["DrugName"]
                    + "</a>"
                )
            count += 1  # Next drug
    return ", ".join(result_list)


def get_drug_summary(drugs):
    red = orange = green = 0
    for x in list(drugs.values()):
        if x["ColourCode"] == "Red":
            red += 1
        if x["ColourCode"] == "Orange":
            orange += 1
        if x["ColourCode"] == "Green":
            green += 1
    if red > 3:
        rslt = (
            "Additional data on "
            + str(red - 3) + " high significance drugs."
            + 'can be found <a href="https://www.spotrm.com">here</a>'
        )
        return rslt
    elif orange > 0:
        rslt = (
            "Additional data on "
            + str(orange) + " significant drugs, "
            + 'can be found <a href="https://www.spotrm.com">here</a>'
        )
        return rslt
    elif green > 0:
        rslt = (
            "Additional data on "
            + str(green) + " other drugs, "
            + 'can be found <a href="https://www.spotrm.com">here</a>'
        )
        return rslt
    else:
        return "No additional data available from SpotRM Web"


def get_spotrm_alerts(dataset, appui):
    # Open the dataset
    ds = app.Dataset()
    ds.parse(dataset)

    smiles_list = smilesList(ds)
    id_list = appui.getSelectedRows(ds)
    if len(id_list) == 0:
        id_list = list(range(len(smiles_list)))
    max_length = 50
    list_length = len(smiles_list)
    if list_length == 0:
        appui.showError("No molecules available")
        return "No molecules available", -1, False
    elif len(id_list) > max_length:
        w = PythonQt.Warning()
        w.setWindowTitle("Warning!")
        w.setMessage(
            "Searching for reactive metabolite alerts for more than "
            + str(max_length)
            + " molecules may take some time.\n"
            + "Do you want to proceed?"
        )
        if not w.run():
            return "Search canceled", -1, False

    global spotrm
    try:
        spotrm
    except NameError:
        try:
            spotrm = SpotRM(appui)
        except Exception as e:
            raise e

    insert_mode = False
    if ds.columnIndex("SpotRM Alerts (Severity Score)") > -1:
        insert_mode = True
        new_entries = ds.getColumn(ds.columnIndex("SpotRM Alerts (Severity Score)"))
        drug_list = ds.getColumn(ds.columnIndex("SpotRM Alerts (Severity Score)") + 1)
        other_data = ds.getColumn(ds.columnIndex("SpotRM Alerts (Severity Score)") + 2)
        severities = ds.getColumn(ds.columnIndex("SpotRM Alerts (Severity Score)") + 3)
    else:
        new_entries = []
        drug_list = []
        other_data = []
        severities = []
        for j in range(list_length):
            drug_list.append(app.Dataset.string(" "))
            new_entries.append(app.Dataset.string(" "))
            other_data.append(app.Dataset.string(" "))
            severities.append(app.Dataset.cont(0.0, 0.0, ""))
    progDlg = PythonQt.SimpleProgress()
    progDlg.modal = True
    progDlg.setWindowTitle("Find SpotRM Alerts")
    progDlg.setMessage("Running SpotRM Alert Search")
    progDlg.setMaximum(len(smiles_list))
    progDlg.setProgress(0)
    progDlg.show()
    appui.updateUI()
    for i, smiles in enumerate(smiles_list):
        if i in id_list:
            try:
                alerts = spotrm.getAlerts(smiles)
                # Add concatenated alerts for each smiles to results
                if not alerts:
                    new_entries[i] = app.Dataset.string(
                            "No alerts found"
                        )
                elif isinstance(alerts, dict):
                    print(str(alerts))
                    new_entries[i] = app.Dataset.string(
                            "Error in SMILES"
                        )
                else:
                    if len(alerts) > 3:
                        new_entries[i] = app.Dataset.string(
                                " || ".join([(x[1] + " (" + x[2] + ")") for x in alerts[0:3]])
                                + ' || + ' + str(len(alerts)) + ' further alerts'
                            )
                    else:
                        new_entries[i] = app.Dataset.string(
                                " || ".join([(x[1] + " (" + x[2] + ")") for x in alerts[0:3]])
                            )
            except Exception as e:
                print("exception in getting alert " + repr(e))
                new_entries[i] = app.Dataset.string("Error retrieving alert")
            try:
                # For each alert retrieve the associated drugs
                result_list = []
                other_list = []
                severity = 0
                a = 0
                for alert in alerts:
                    if a < 3:
                        if alert == 'error':
                            result_list.append("-")
                            other_list.append("-")
                            break
                        else:
                            drugs = spotrm.getDrug(alert[0])
                            result_list.append(get_drug_links(drugs))
                            other_list.append(get_drug_summary(drugs))
                        a += 1
                    else:
                        break
                for alert in alerts:
                    if alert != 'error':
                        severity += int(alert[2])
                drug_list[i] = app.Dataset.string(" || ".join(result_list))
                other_data[i] = app.Dataset.string(" || ".join(other_list))
                severities[i] = app.Dataset.cont(float(severity), 0.0, "")
                progDlg.setProgress(i)
                appui.updateUI()
            except Exception as e:
                print("exception in finding drug " + repr(e))
                drug_list[i] = app.Dataset.string("unknown")
        progDlg.setProgress(i)
        appui.updateUI()

    progDlg.hide()
    progDlg.delete()

    alert_column = app.Dataset.meta("SpotRM Alerts (Severity Score)", app.string)
    severity_column = app.Dataset.meta(
        "SpotRM Severity Sum", app.cont, ((0.0, 100.0), (0, 0))
    )
    drug_column = app.Dataset.meta("SpotRM Top 3 Drug References", app.string)
    other_column = app.Dataset.meta("SpotRM other data available", app.string)
    if ds.columnIndex("SpotRM Alerts (Severity Score)") > -1:
        structure_column_index = ds.columnIndex("SpotRM Alerts (Severity Score)") - 1
        updating = True
    else:
        structure_column_index = structureColumnIndex(ds)
        updating = False
    ds.insertColumn(alert_column, new_entries, structure_column_index + 1)
    ds.insertColumn(drug_column, drug_list, structure_column_index + 2)
    ds.insertColumn(other_column, other_data, structure_column_index + 3)
    ds.insertColumn(severity_column, severities, structure_column_index + 4)
    return (
        ds.packColumn(structure_column_index + 1),
        ds.packColumn(structure_column_index + 2),
        ds.packColumn(structure_column_index + 3),
        ds.packColumn(structure_column_index + 4),
        structure_column_index + 1,
        updating,
    )


def get_spotrm_alerts_for_dataset(appui):
    dataset = appui.importDS("")
    # Search for Alerts
    try:
        alerts, drugs, other, severities, index, update = get_spotrm_alerts(dataset, appui)
    except Exception as e:
        print("Found exception" + repr(e))
        return
    if update:
        appui.removeColumnFromDataSet("", index)
        appui.addColumnToDataSet("SpotRM Alerts (Severity Score)", index, alerts)
        appui.removeColumnFromDataSet("", index + 1)
        appui.addColumnToDataSet("SpotRM Top 3 Drug References", index + 1, drugs)
        appui.removeColumnFromDataSet("", index + 2)
        appui.addColumnToDataSet("SpotRM other data available", index + 2, other)
        appui.removeColumnFromDataSet("", index + 3)
        appui.addColumnToDataSet("SpotRM Severity Sum", index + 3, severities)
    else:
        appui.addColumnToDataSet("SpotRM Alerts (Severity Score)", index, alerts)
        appui.addColumnToDataSet("SpotRM Top 3 Drug References", index + 1, drugs)
        appui.addColumnToDataSet("SpotRM other data available", index + 2, other)
        appui.addColumnToDataSet("SpotRM Severity Sum", index + 3, severities)


def get_stardrop_definitions():
    definitions = [
        {
            "script_name": "SpotRM analysis",
            "callback": get_spotrm_alerts_for_dataset,
        }
    ]

    return definitions
